package eu.dnetlib.irishmonitorservice.services;

import eu.dnetlib.irishmonitorservice.configuration.properties.StatsToolProperties;
import eu.dnetlib.irishmonitorservice.entities.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.util.UriComponentsBuilder;

import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.net.URLDecoder;
import java.util.List;

@Service
public class StatsToolService {

    @Autowired
    private StatsToolProperties properties;

    @Autowired
    private RestTemplate restTemplate;

    @Cacheable(value = "funders")
    public List<List<String>> getFunders() throws UnsupportedEncodingException {
        String url = this.properties.getRfo();
        return this.getData(url);
    }

    @Cacheable(value = "organizations")
    public List<List<String>> getOrganizations() throws UnsupportedEncodingException {
        String url = this.properties.getRpo();
        return this.getData(url);
    }

    @Cacheable(value = "datasources")
    public List<List<String>> getDataSources() throws UnsupportedEncodingException {
        String url = this.properties.getRepository();
        return this.getData(url);
    }

    private List<List<String>> getData(String url) throws UnsupportedEncodingException {
        if(url != null) {
            String service = url.substring(0, url.indexOf('=') + 1);
            String json = URLDecoder.decode(url.substring(url.indexOf('=') + 1), "UTF-8");
            URI uri = UriComponentsBuilder.fromUriString(service + json).build().encode().toUri();
            try {
                ResponseEntity<Data> data = restTemplate.getForEntity(uri, Data.class);
                if(data.getStatusCode() == HttpStatus.OK) {
                    return data.getBody().getData();
                }
            } catch (RestClientException e) {
                return null;
            }
        }
        return null;
    }
}
